// BeeHive - Dynamic Menu, August 14, 2002
// --------------------------------------------------------
// (c) 2002, Peter Nederlof, http://www.xs4all.nl/~peterned

// To do list
// --------------------------------------
// - Position calc. control, +performance
// - Full style-inheritance for submenus
// - Custom opening directions for submenus
// - (maybe) Cascading drag behaviour
// - Animated (moving) menus
// - (maybe) across frames

var menuSettings = {
	menuWidth:140,
	menuOpacity:100,
	menuXOffset:0,
	menuYOffset:0,
	menuDragging:false,
	menuStyle:'font-family:tahoma; font-size:8pt; font-weight:bold;',
	frameTarget:false,
	borderWidth:1,
	borderColor:'#204080',
	itemSpacing:0,
	itemPadding:3,
	itemSubMarker:'images/subarrow.gif',
	itemIcon:'<img src="images/file.gif" width=15 height=15> ',
	itemBgImage:false,
	itemBgColor:'#30d0ff',
	itemBgHighColor:'#2080c0',
	itemFgColor:'#000000',
	itemFgHighColor:'#ffffff',
	titleBgColor:'#20a0e0',
	titleFgColor:'#000000',
	disabledFontColor:'#1090d0',
	set:function(arg, to) {this[arg] = to;}
}

var dynMenus = [];
var menuCloser = false;
var inItem = null;

// Initialise menu
// ----------------------------------------------
document.attachEvent("onload", "initDynMenu()");
function initDynMenu() {
	for(var i=0; i<dynMenus.length; i++) {
		var menu = dynMenus[i]
		menu.build();
		
		if(menu.menuOpacity < 100)
				menu.menuLayer.setOpacity(menu.menuOpacity)
		if(menu.id.indexOf('menu') > -1) {
			menu.menuLayer.setVisible(true);
			if(menu.menuDragging)
				menu.items[0].itemLayer.enableDragDrop('dragParent bringToFront');			
		}	
	}	
}

// Menu Object
// ----------------------------------------------
function menu(x, y, nest, id) {
	this.id = id || ("menu" + dynMenus.length);
	this.layerId = this.id + "Lyr";
	this.items = [];
	this.children = [];
	this.nestIn = nest;
	this.xPos = x || 10;
	this.yPos = y || 10;

	for(var i in menuSettings) {
		this[i] = menuSettings[i];
	}
	
	dynMenus[dynMenus.length] = this;
	dynMenus[this.id] = this;
}
	
menuProto = menu.prototype;
menuProto.addItem = function(label, link, subId, title) {
	var pos = this.items.length;
	this.items[pos] = new mnItem(label, link, this.id+pos);
	this.items[pos].parent = this;
	this.items[pos].child = subId || false;
	this.items[pos].isSubmenu = subId? true:false;
	this.items[pos].isTitle = title || false;
	this.children[this.children.length] = subId || false;
}

menuProto.addSubmenu = function(label) {
	var subMn = 'subMn' + dynMenus.length;
	this.addItem(label, null, subMn);
	return new menu(0,0,this.nestIn,subMn);		
}

menuProto.addTitle = function(label) {
	this.addItem(label, null, null, true);
}

menuProto.build = function() {
	this.itemWidth = this.menuWidth - 2*this.borderWidth;
	with(this.menuLayer = new dynObject(this.layerId, this.nestIn)) {
		setProperties(this.xPos, this.yPos, this.menuWidth, 100,"hidden", this.borderColor,400);			
	}

	var end = 0;
	for(var i=0; i<this.items.length; i++) {
		this.items[i].build(); end = i;
		if(i != 0) this.items[i].position(i);
		var iChild = this.children[i];
		if(iChild) this.children[i] = dynMenus[iChild];
		
		with(this.items[i].itemLayer) {
			getDimensions();
			resizeTo(this.itemWidth, h + this.itemPadding);
		}
	}

	var endItem = this.items[end].itemLayer;
	this.menuLayer.resizeTo(this.menuWidth, endItem.b + this.borderWidth);
}

menuProto.moveTo = function(x,y) {
	this.menuLayer.moveTo(x,y);
}

// Menuitem Object
// ----------------------------------------------

function mnItem(label, link, lId) {
	this.id = label;
	this.layerId = "itLyr" + lId;
	this.label = label;
	this.link = link;
	this.isTitle = false;
	this.isSubmenu = false;	
}
mniProto = mnItem.prototype
mniProto.build = function(y) {
	with(this.parent) {
		var useBgColor = this.isTitle? titleBgColor:itemBgColor;
		var useFgColor = this.isTitle? titleFgColor:((!this.link && !this.isSubmenu)? disabledFontColor:itemFgColor);
		this.itemLayer = new dynObject(this.layerId, layerId);
		this.itemLayer.setProperties(borderWidth, borderWidth, itemWidth, null, "hidden", useBgColor);
		var useIcon = this.isTitle? "":(itemIcon? itemIcon:'');
	}

	with(this.itemLayer) {
		css.visibility = "inherit";
		css.textAlign = 'left'
		write('<span style="'+this.parent.menuStyle+((is.NS4)? 'color:'
			+useFgColor:'')+'">'+useIcon+'&nbsp;'+this.label+'</span>');
		css.color = useFgColor;
	}

	if(this.parent.itemBgImage && !this.isTitle) {
		this.itemLayer.setBackground(this.parent.itemBgImage);
	}

	if(this.isSubmenu && this.parent.itemSubMarker) {
		with(this.subLayer = new dynObject(this.layerId + 'mark', this.layerId)) {
			setProperties(this.parent.itemWidth - 9,5,20,20,"hidden");
			createImage(this.parent.itemSubMarker,0,5,9); 
			css.visibility = "inherit";
		}
	} else {
		if(!this.isTitle && !this.link)
			this.itemLayer.css.color = this.parent.disabledFontColor;
	}

	this.itemLayer.parentItem = this;
	if(this.child) this.child = dynMenus[this.child];
	
	with(this.parent) {
		this.itemLayer.attachEvent("onmouseover", "mnItemOver()");
		if(this.isTitle || (!this.isSubmenu && !this.link)) return;
		this.itemLayer.attachEvent("onmouseover","mnItemHighlight('"+itemBgHighColor+"', '"+itemFgHighColor+"')")
		this.itemLayer.attachEvent("onmouseout",  "mnItemOut('"+itemBgColor+"', '"+itemFgColor+"')");
		if(!this.isSubmenu) this.itemLayer.attachEvent("onmousedown", "mnItemExecute()");
	}
}

mniProto.position = function mnItemPosition(i) {
	with(this.parent) {
		var above = items[i-1].itemLayer;
		var y = above.b + itemSpacing;
		this.itemLayer.moveTo(borderWidth ,y);
	}
}

// Menu behaviour
// ----------------------------------------------
function mnItemHighlight(color, fColor) {
	activeObj.setBackground(color);
	activeObj.css.color = fColor;
}

function mnItemOver() {
	clearTimeout(menuCloser);
	inItem = activeObj.parentItem;
	var menu = inItem.parent;
	
	if(menu.id.indexOf('menu') > -1) hideAllMenus();
	else hideChildren(menu);

	if(inItem.child && !inItem.child.menuLayer.visible) {
		var hOffset = menu.menuLayer.w;
		if(menu.menuLayer.x + 2*hOffset > doc.availWidth) hOffset *= -0.8;
		x = hOffset + menu.menuXOffset + menu.menuLayer.x - menu.borderWidth;
		y = menu.menuLayer.y + menu.menuYOffset + inItem.itemLayer.y - menu.borderWidth;
		inItem.child.moveTo(x, y);
		with(inItem.child.menuLayer) {
			setZIndex(menu.menuLayer.z + 1)
			setVisible(true);
		}
	}
}
	function hideChildren(obj) {
		for(var i=0; i<obj.children.length; i++) {
			var useObj = obj.children[i];
			if(useObj) {
				hideChildren(useObj)
				useObj.menuLayer.setVisible(false);
			}
		}
	}

	function hideAllMenus() {
		for(var i=0; i<dynMenus.length; i++) {
			if(dynMenus[i].id.indexOf('menu') > -1) {
				hideChildren(dynMenus[i]);
			}
		}
	}

function mnItemOut(color, fColor) {
	activeObj.setBackground(color);
	activeObj.css.color = fColor;
	clearTimeout(menuCloser);
	menuCloser = setTimeout('hideAllMenus()', 1000);
}

function mnItemExecute() {
	var frame = activeObj.parentItem.parent.frameTarget;
	var target = (frame)?
		parent.frames[frame].document:document;	
	target.location.href = inItem.link;
}